"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.corewar = void 0;
const IMessage_1 = require("./simulator/interface/IMessage");
const Parser_1 = require("./parser/Parser");
const Scanner_1 = require("./parser/Scanner");
const ForPass_1 = require("./parser/ForPass");
const PreprocessCollector_1 = require("./parser/PreprocessCollector");
const PreprocessAnalyser_1 = require("./parser/PreprocessAnalyser");
const PreprocessEmitter_1 = require("./parser/PreprocessEmitter");
const LabelCollector_1 = require("./parser/LabelCollector");
const LabelEmitter_1 = require("./parser/LabelEmitter");
const MathsProcessor_1 = require("./parser/MathsProcessor");
const Expression_1 = require("./parser/Expression");
const Filter_1 = require("./parser/Filter");
const MetaDataCollector_1 = require("./parser/MetaDataCollector");
const DefaultPass_1 = require("./parser/DefaultPass");
const OrgPass_1 = require("./parser/OrgPass");
const SyntaxCheck_1 = require("./parser/SyntaxCheck");
const LoadFileSerialiser_1 = require("./parser/LoadFileSerialiser");
const IllegalCommandCheck_1 = require("./parser/IllegalCommandCheck");
const MetaDataEmitter_1 = require("./parser/MetaDataEmitter");
const Random_1 = require("./simulator/Random");
const Executive_1 = require("./simulator/Executive");
const Decoder_1 = require("./simulator/Decoder");
const Core_1 = require("./simulator/Core");
const Loader_1 = require("./simulator/Loader");
const WarriorLoader_1 = require("./simulator/WarriorLoader");
const Fetcher_1 = require("./simulator/Fetcher");
const Simulator_1 = require("./simulator/Simulator");
const EndCondition_1 = require("./simulator/EndCondition");
const OptionValidator_1 = require("./simulator/OptionValidator");
const Publisher_1 = require("./simulator/Publisher");
const LatestOnlyStrategy_1 = require("./simulator/LatestOnlyStrategy");
const PerKeyStrategy_1 = require("./simulator/PerKeyStrategy");
const MatchRunner_1 = require("./matches/MatchRunner");
const MatchResultMapper_1 = require("./matches/MatchResultMapper");
const HillRunner_1 = require("./matches/HillRunner");
const HillResultMapper_1 = require("./matches/HillResultMapper");
const BenchmarkRunner_1 = require("./matches/BenchmarkRunner");
const clone = require("clone");
class Api {
    constructor() {
        // any setup needed for the NPM package to work properly
        // like creating the simulator and parser
        const expression = new Expression_1.Expression();
        this.serialiser = new LoadFileSerialiser_1.LoadFileSerialiser();
        this.parser = new Parser_1.Parser(new Scanner_1.Scanner(), new Filter_1.Filter(), new MetaDataCollector_1.MetaDataCollector(), new ForPass_1.ForPass(expression), new PreprocessCollector_1.PreprocessCollector(), new PreprocessAnalyser_1.PreprocessAnalyser(), new PreprocessEmitter_1.PreprocessEmitter(), new LabelCollector_1.LabelCollector(), new LabelEmitter_1.LabelEmitter(), new MathsProcessor_1.MathsProcessor(expression), new DefaultPass_1.DefaultPass(), new OrgPass_1.OrgPass(), new SyntaxCheck_1.SyntaxCheck(), new IllegalCommandCheck_1.IllegalCommandCheck(), new MetaDataEmitter_1.MetaDataEmitter());
        this.publisher = new Publisher_1.Publisher({
            [IMessage_1.MessageType.CoreAccess]: new PerKeyStrategy_1.PerKeyStrategy(p => p.address),
            [IMessage_1.MessageType.RunProgress]: new LatestOnlyStrategy_1.LatestOnlyStrategy(),
            [IMessage_1.MessageType.RoundEnd]: new LatestOnlyStrategy_1.LatestOnlyStrategy(),
            [IMessage_1.MessageType.TaskCount]: new PerKeyStrategy_1.PerKeyStrategy(p => p.warriorId),
            [IMessage_1.MessageType.CoreInitialise]: new LatestOnlyStrategy_1.LatestOnlyStrategy(),
            [IMessage_1.MessageType.RoundStart]: new LatestOnlyStrategy_1.LatestOnlyStrategy(),
            [IMessage_1.MessageType.NextExecution]: new LatestOnlyStrategy_1.LatestOnlyStrategy(),
            [IMessage_1.MessageType.MatchEnd]: new LatestOnlyStrategy_1.LatestOnlyStrategy(),
            [IMessage_1.MessageType.HillEnd]: new LatestOnlyStrategy_1.LatestOnlyStrategy(),
            [IMessage_1.MessageType.WarriorDead]: new LatestOnlyStrategy_1.LatestOnlyStrategy()
        });
        this.core = new Core_1.Core(this.publisher);
        const loader = new Loader_1.Loader(new Random_1.Random(), this.core, new WarriorLoader_1.WarriorLoader(this.core, this.publisher));
        const fetcher = new Fetcher_1.Fetcher();
        this.executive = new Executive_1.Executive(this.publisher);
        const decoder = new Decoder_1.Decoder(this.executive);
        this.simulator = new Simulator_1.Simulator(this.core, loader, fetcher, decoder, this.executive, new EndCondition_1.EndCondition(this.publisher), new OptionValidator_1.OptionValidator(), this.publisher);
        this.matchRunner = new MatchRunner_1.MatchRunner(this.simulator, new MatchResultMapper_1.MatchResultMapper(), this.publisher);
        this.hillRunner = new HillRunner_1.HillRunner(this.publisher, this.matchRunner, new HillResultMapper_1.HillResultMapper());
        this.benchmarkRunner = new BenchmarkRunner_1.BenchmarkRunner(this.publisher, this.matchRunner, new HillResultMapper_1.HillResultMapper());
    }
    initialiseSimulator(options, warriors, messageProvider) {
        // TODO move this into library and unit test
        // TODO perform this check on run match, hill and benchmark also
        if (warriors.filter(warrior => !warrior.source.success).length) {
            return;
        }
        if (messageProvider) {
            this.publisher.setPublishProvider(messageProvider);
        }
        this.executive.initialise(options);
        this.simulator.initialise(options, warriors);
    }
    step(steps) {
        return clone(this.simulator.step(steps));
    }
    run() {
        return clone(this.simulator.run());
    }
    parse(redcode) {
        return clone(this.parser.parse(redcode));
    }
    serialise(tokens) {
        return this.serialiser.serialise(tokens);
    }
    //TODO I think we should rename this to getAt
    getWithInfoAt(address) {
        return clone(this.core.getWithInfoAt(address));
    }
    republish() {
        this.publisher.republish();
    }
    runMatch(rules, warriors, messageProvider) {
        if (messageProvider) {
            this.publisher.setPublishProvider(messageProvider);
        }
        return clone(this.matchRunner.run(rules, warriors));
    }
    runHill(rules, warriors, messageProvider) {
        if (messageProvider) {
            this.publisher.setPublishProvider(messageProvider);
        }
        return clone(this.hillRunner.run(rules, warriors));
    }
    runBenchmark(warrior, rules, warriors, messageProvider) {
        this.publisher.setPublishProvider(messageProvider);
        return clone(this.benchmarkRunner.run(warrior, rules, warriors));
    }
}
// exports for use in npm package
exports.corewar = new Api();
//# sourceMappingURL=index.js.map