"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OrgPass = void 0;
const IToken_1 = require("./interface/IToken");
const IParseOptions_1 = require("./interface/IParseOptions");
const PassBase_1 = require("./PassBase");
class OrgPass extends PassBase_1.PassBase {
    process(context, options) {
        // Replace END ### with ORG ###
        // Emit ORG as first instruction
        // Raise warning for duplicate ORGs / END ###
        // Under ICWS'86 - if no END ### found, if start label defined, emit ORG start
        this.firstInstruction = null;
        this.org = null;
        this.orgAddress = null;
        this.orgComment = null;
        const result = super.process(context, options);
        this.emitOrg();
        return result;
    }
    processLine() {
        const next = this.stream.peek();
        if (this.firstInstruction === null && next.category !== IToken_1.TokenCategory.Comment) {
            this.firstInstruction = this.stream.position;
        }
        if (next.category === IToken_1.TokenCategory.Preprocessor) {
            if (next.lexeme === 'ORG') {
                this.processOrg();
            }
            else if (next.lexeme === 'END') {
                this.processEnd();
            }
            else {
                this.context.emit(this.stream.readToEOL());
            }
        }
        else {
            this.context.emit(this.stream.readToEOL());
        }
    }
    processOrg() {
        const org = this.stream.expectOnly('ORG');
        this.org = org;
        if (this.orgAddress !== null) {
            this.stream.warn(org, 'Redefinition of ORG encountered, this later definition will take effect');
        }
        const address = this.stream.expect(IToken_1.TokenCategory.Number);
        this.orgAddress = parseInt(address.lexeme, 10);
        if (this.stream.peek().category === IToken_1.TokenCategory.Comment) {
            this.orgComment = this.stream.read();
        }
        this.stream.expect(IToken_1.TokenCategory.EOL);
    }
    processEnd() {
        const end = this.stream.expectOnly('END');
        let address = null;
        if (this.stream.peek().category === IToken_1.TokenCategory.Number) {
            address = this.stream.read();
        }
        if (this.stream.peek().category === IToken_1.TokenCategory.Comment) {
            this.stream.read();
        }
        this.stream.expect(IToken_1.TokenCategory.EOL);
        if (address !== null) {
            if (this.orgAddress !== null) {
                this.stream.warn(end, 'Encountered both ORG and END address, the ORG definition will take effect');
            }
            else {
                this.org = end;
                this.orgAddress = parseInt(address.lexeme, 10);
            }
        }
    }
    emitOrg() {
        if (this.orgAddress === null) {
            if (this.options.standard === IParseOptions_1.Standard.ICWS86 &&
                Object.keys(this.context.labels).includes(OrgPass.START_LABEL)) {
                this.orgAddress = this.context.labels[OrgPass.START_LABEL];
            }
            else {
                this.orgAddress = 0;
            }
            this.org = {
                category: IToken_1.TokenCategory.Preprocessor,
                lexeme: 'ORG',
                position: { line: 1, char: 1 }
            };
        }
        const org = {
            category: IToken_1.TokenCategory.Preprocessor,
            lexeme: 'ORG',
            position: Object.assign({}, this.org.position)
        };
        const address = {
            category: IToken_1.TokenCategory.Number,
            lexeme: this.orgAddress.toString(),
            position: Object.assign({}, this.org.position)
        };
        const instruction = [org, address];
        if (this.orgComment !== null) {
            instruction.push(this.orgComment);
        }
        instruction.push({
            category: IToken_1.TokenCategory.EOL,
            lexeme: '\n',
            position: Object.assign({}, this.org.position)
        });
        this.context.tokens.splice(this.firstInstruction, 0, ...instruction);
    }
}
exports.OrgPass = OrgPass;
OrgPass.START_LABEL = 'start';
//# sourceMappingURL=OrgPass.js.map