"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PreprocessCollector = void 0;
const IParseOptions_1 = require("./interface/IParseOptions");
const IToken_1 = require("./interface/IToken");
const IMessage_1 = require("./interface/IMessage");
const PassBase_1 = require("./PassBase");
class PreprocessCollector extends PassBase_1.PassBase {
    /// <summary>
    /// Records EQU substitutions and removes statements from token stream
    /// Performs a duplicate label check
    /// </summary>
    process(context, options) {
        // Record EQU label tokens
        // Remove EQU token labels from token stream
        // Duplicate label check
        this.previous = [];
        return super.process(context, options);
    }
    processLine() {
        while (!this.stream.eof()) {
            const next = this.stream.peek();
            if (next.category === IToken_1.TokenCategory.Label) {
                this.previous = [];
                this.processLabels();
            }
            else if (this.isMultilineEqu(next)) {
                this.processMultilineEqu();
            }
            else {
                const line = this.stream.readToEOL();
                this.context.emit(line);
            }
        }
    }
    isMultilineEqu(next) {
        return (next.category === IToken_1.TokenCategory.Preprocessor &&
            next.lexeme === 'EQU' &&
            this.previous.length > 0 &&
            this.options.standard === IParseOptions_1.Standard.ICWS94draft);
    }
    isEqu(pre) {
        return pre.category === IToken_1.TokenCategory.Preprocessor && pre.lexeme === 'EQU';
    }
    processLabels() {
        const labels = [];
        while (this.stream.peek().category === IToken_1.TokenCategory.Label) {
            const token = this.stream.expect(IToken_1.TokenCategory.Label);
            this.previous.push(token.lexeme);
            labels.push(token);
        }
        const pre = this.stream.read();
        if (this.isEqu(pre)) {
            this.processEqu(labels);
        }
        else {
            this.previous = [];
            this.context.emit(labels);
            this.context.emit([pre]);
        }
    }
    warnDuplicateLabel(label) {
        this.context.messages.push({
            type: IMessage_1.MessageType.Warning,
            position: label.position,
            text: "Redefinition of label '" + label.lexeme + "', original definition will be used"
        });
    }
    processEqu(labels) {
        let expression = this.stream.readToEOL();
        // Do not include terminating EOL in replacement expression
        expression.pop();
        // Remove comments
        expression = expression.filter((token) => {
            return token.category !== IToken_1.TokenCategory.Comment;
        });
        labels.forEach((label) => {
            if (label.lexeme in this.context.equs) {
                this.warnDuplicateLabel(label);
            }
            else {
                this.context.equs[label.lexeme] = expression;
            }
        });
    }
    processMultilineEqu() {
        this.stream.expectOnly('EQU');
        let expression = [
            {
                category: IToken_1.TokenCategory.EOL,
                lexeme: '\n',
                position: Object.assign({}, this.stream.peek().position)
            }
        ];
        expression = expression.concat(this.stream.readToEOL());
        // Remove terminating newline
        expression.pop();
        this.previous.forEach((label) => {
            const existing = this.context.equs[label];
            this.context.equs[label] = existing.concat(expression);
        });
    }
}
exports.PreprocessCollector = PreprocessCollector;
//# sourceMappingURL=PreprocessCollector.js.map