"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Decoder = void 0;
const IOperand_1 = require("./interface/IOperand");
const IInstruction_1 = require("./interface/IInstruction");
const clone = require("clone");
class Decoder {
    constructor(executive) {
        this.deferredSets = [];
        this.modeTable = {
            [IOperand_1.ModeType.Immediate]: this.immediate,
            [IOperand_1.ModeType.Direct]: this.direct,
            [IOperand_1.ModeType.AIndirect]: this.aIndirect,
            [IOperand_1.ModeType.BIndirect]: this.bIndirect,
            [IOperand_1.ModeType.APreDecrement]: this.aPreDecrement,
            [IOperand_1.ModeType.BPreDecrement]: this.bPreDecrement,
            [IOperand_1.ModeType.APostIncrement]: this.aPostIncrement,
            [IOperand_1.ModeType.BPostIncrement]: this.bPostIncrement // >
        };
        this.executive = executive;
    }
    decode(context) {
        const aAccessor = this.modeTable[context.instruction.aOperand.mode];
        const bAccessor = this.modeTable[context.instruction.bOperand.mode];
        this.deferredSets = [];
        context.aInstruction = aAccessor.apply(this, [
            context.task,
            context.instructionPointer,
            context.instruction.aOperand,
            context.core
        ]);
        context.bInstruction = bAccessor.apply(this, [
            context.task,
            context.instructionPointer,
            context.instruction.bOperand,
            context.core
        ]);
        context.command = this.executive.commandTable[context.instruction.opcode];
        context.operands = this.decodeModifier(context);
        //HACK only write to core as a result of decoding AFTER all decoding has finished
        // in order to emulate PMARS
        this.deferredSets.forEach(ds => {
            context.core.setAt(context.task, ds.address, ds.instruction);
        });
        return context;
    }
    decodeModifier(context) {
        switch (context.instruction.modifier) {
            case IInstruction_1.ModifierType.A:
                return [
                    {
                        source: context.aInstruction.aOperand,
                        destination: context.bInstruction.aOperand
                    }
                ];
            case IInstruction_1.ModifierType.B:
                return [
                    {
                        source: context.aInstruction.bOperand,
                        destination: context.bInstruction.bOperand
                    }
                ];
            case IInstruction_1.ModifierType.AB:
                return [
                    {
                        source: context.aInstruction.aOperand,
                        destination: context.bInstruction.bOperand
                    }
                ];
            case IInstruction_1.ModifierType.BA:
                return [
                    {
                        source: context.aInstruction.bOperand,
                        destination: context.bInstruction.aOperand
                    }
                ];
            case IInstruction_1.ModifierType.F:
                return [
                    {
                        source: context.aInstruction.aOperand,
                        destination: context.bInstruction.aOperand
                    },
                    {
                        source: context.aInstruction.bOperand,
                        destination: context.bInstruction.bOperand
                    }
                ];
            case IInstruction_1.ModifierType.X:
                return [
                    {
                        source: context.aInstruction.aOperand,
                        destination: context.bInstruction.bOperand
                    },
                    {
                        source: context.aInstruction.bOperand,
                        destination: context.bInstruction.aOperand
                    }
                ];
            case IInstruction_1.ModifierType.I:
                return [
                    {
                        source: context.aInstruction.aOperand,
                        destination: context.bInstruction.aOperand
                    },
                    {
                        source: context.aInstruction.bOperand,
                        destination: context.bInstruction.bOperand
                    }
                ];
            default:
                throw 'Unknown modifier: ' + context.instruction.modifier;
        }
    }
    immediate(_, ip, __, core) {
        const address = ip;
        return clone(core.getAt(address));
    }
    direct(_, ip, operand, core) {
        const address = ip + operand.address;
        return clone(core.getAt(address));
    }
    aIndirect(task, ip, operand, core) {
        const ipa = ip + operand.address;
        const address = ipa + core.readAt(task, ipa).aOperand.address;
        return clone(core.getAt(address));
    }
    bIndirect(task, ip, operand, core) {
        const ipa = ip + operand.address;
        const address = ipa + core.readAt(task, ipa).bOperand.address;
        return clone(core.getAt(address));
    }
    aPreDecrement(task, ip, operand, core) {
        const ipa = ip + operand.address;
        const instruction = clone(core.readAt(task, ipa));
        let value = instruction.aOperand.address;
        const address = ipa + --value;
        instruction.aOperand.address = core.wrap(value);
        this.deferredSets.push({
            address: ipa,
            instruction
        });
        return clone(core.getAt(address));
    }
    bPreDecrement(task, ip, operand, core) {
        const ipa = ip + operand.address;
        const instruction = core.readAt(task, ipa);
        let value = instruction.bOperand.address;
        const address = ipa + --value;
        instruction.bOperand.address = core.wrap(value);
        this.deferredSets.push({
            address: ipa,
            instruction
        });
        return clone(core.getAt(address));
    }
    aPostIncrement(task, ip, operand, core) {
        const ipa = ip + operand.address;
        const instruction = core.readAt(task, ipa);
        let value = instruction.aOperand.address;
        const address = ipa + value++;
        const result = clone(core.getAt(address));
        instruction.aOperand.address = core.wrap(value);
        this.deferredSets.push({
            address: ipa,
            instruction
        });
        return result;
    }
    bPostIncrement(task, ip, operand, core) {
        const ipa = ip + operand.address;
        const instruction = core.readAt(task, ipa);
        let value = instruction.bOperand.address;
        const address = ipa + value++;
        const result = clone(core.getAt(address));
        instruction.bOperand.address = core.wrap(value);
        this.deferredSets.push({
            address: ipa,
            instruction
        });
        return result;
    }
}
exports.Decoder = Decoder;
//# sourceMappingURL=Decoder.js.map