"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Executive = void 0;
const IInstruction_1 = require("./interface/IInstruction");
const IMessage_1 = require("./interface/IMessage");
class Executive {
    constructor(publisher) {
        this.commandTable = {
            [IInstruction_1.OpcodeType.DAT]: this.dat,
            [IInstruction_1.OpcodeType.MOV]: this.mov,
            [IInstruction_1.OpcodeType.ADD]: this.add,
            [IInstruction_1.OpcodeType.SUB]: this.sub,
            [IInstruction_1.OpcodeType.MUL]: this.mul,
            [IInstruction_1.OpcodeType.DIV]: this.div,
            [IInstruction_1.OpcodeType.MOD]: this.mod,
            [IInstruction_1.OpcodeType.JMP]: this.jmp,
            [IInstruction_1.OpcodeType.JMZ]: this.jmz,
            [IInstruction_1.OpcodeType.JMN]: this.jmn,
            [IInstruction_1.OpcodeType.DJN]: this.djn,
            [IInstruction_1.OpcodeType.CMP]: this.seq,
            [IInstruction_1.OpcodeType.SEQ]: this.seq,
            [IInstruction_1.OpcodeType.SNE]: this.sne,
            [IInstruction_1.OpcodeType.SLT]: this.slt,
            [IInstruction_1.OpcodeType.SPL]: this.spl,
            [IInstruction_1.OpcodeType.NOP]: this.nop
        };
        this.publisher = publisher;
    }
    initialise(options) {
        this.maxTasks = options.maxTasks;
    }
    publishTaskCount(instance, taskCount) {
        const payload = {
            warriorId: instance.warrior.internalId,
            taskCount
        };
        if (instance.warrior.data) {
            payload['warriorData'] = instance.warrior.data;
        }
        this.publisher.queue({
            type: IMessage_1.MessageType.TaskCount,
            payload
        });
        if (taskCount === 0) {
            this.publisher.queue({
                type: IMessage_1.MessageType.WarriorDead,
                payload
            });
        }
    }
    dat(context) {
        //Remove current task from the queue
        const ti = context.taskIndex;
        context.instance.tasks.splice(ti, 1);
        // wrap the warrior task index to cater for the event when
        // we just chomped off the last task
        context.instance.taskIndex = ti % context.instance.tasks.length;
        this.publishTaskCount(context.instance, context.instance.tasks.length);
    }
    mov(context) {
        if (context.instruction.modifier === IInstruction_1.ModifierType.I) {
            context.aInstruction.address = context.bInstruction.address;
            context.core.setAt(context.task, context.bInstruction.address, context.aInstruction);
            return;
        }
        context.operands.forEach(p => {
            p.destination.address = p.source.address;
        });
        context.core.setAt(context.task, context.bInstruction.address, context.bInstruction);
    }
    add(context) {
        context.operands.forEach(p => {
            p.destination.address = context.core.wrap(p.destination.address + p.source.address);
        });
        context.core.setAt(context.task, context.bInstruction.address, context.bInstruction);
    }
    sub(context) {
        context.operands.forEach(p => {
            p.destination.address = context.core.wrap(p.destination.address - p.source.address);
        });
        context.core.setAt(context.task, context.bInstruction.address, context.bInstruction);
    }
    mul(context) {
        context.operands.forEach(p => {
            p.destination.address = context.core.wrap(p.destination.address * p.source.address);
        });
        context.core.setAt(context.task, context.bInstruction.address, context.bInstruction);
    }
    div(context) {
        context.operands.forEach(p => {
            if (p.source.address !== 0) {
                p.destination.address = (p.destination.address / p.source.address) >> 0;
            }
            else {
                //TODO double divide by zero will remove two processes from the warrior task queue!
                this.dat(context);
            }
        });
        context.core.setAt(context.task, context.bInstruction.address, context.bInstruction);
    }
    mod(context) {
        context.operands.forEach(p => {
            if (p.source.address !== 0) {
                p.destination.address = p.destination.address % p.source.address;
            }
            else {
                //TODO double divide by zero will remove two processes from the warrior task queue!
                this.dat(context);
            }
        });
        context.core.setAt(context.task, context.bInstruction.address, context.bInstruction);
    }
    jmp(context) {
        context.task.instructionPointer = context.core.wrap(context.aInstruction.address);
    }
    jmz(context) {
        let branch = true;
        context.operands.forEach(p => {
            if (p.destination.address !== 0) {
                branch = false;
            }
        });
        if (branch) {
            context.task.instructionPointer = context.core.wrap(context.aInstruction.address);
        }
    }
    jmn(context) {
        let branch = true;
        context.operands.forEach(p => {
            if (p.destination.address === 0) {
                branch = false;
            }
        });
        if (branch) {
            context.task.instructionPointer = context.core.wrap(context.aInstruction.address);
        }
    }
    djn(context) {
        let branch = false;
        context.operands.forEach(p => {
            const a = context.core.wrap(p.destination.address - 1);
            p.destination.address = a;
            if (a !== 0) {
                branch = true;
            }
        });
        if (branch) {
            context.task.instructionPointer = context.core.wrap(context.aInstruction.address);
        }
        context.core.setAt(context.task, context.bInstruction.address, context.bInstruction);
    }
    seq(context) {
        let branch = true;
        if (context.instruction.modifier === IInstruction_1.ModifierType.I) {
            return this.seqI(context);
        }
        context.operands.forEach(p => {
            if (p.source.address !== p.destination.address) {
                branch = false;
            }
        });
        if (branch) {
            context.task.instructionPointer = context.core.wrap(context.task.instructionPointer + 1);
        }
    }
    seqI(context) {
        const ai = context.aInstruction;
        const bi = context.bInstruction;
        if (ai.opcode === bi.opcode &&
            ai.modifier === bi.modifier &&
            ai.aOperand.mode === bi.aOperand.mode &&
            ai.aOperand.address === bi.aOperand.address &&
            ai.bOperand.mode === bi.bOperand.mode &&
            ai.bOperand.address === bi.bOperand.address) {
            context.task.instructionPointer = context.core.wrap(context.task.instructionPointer + 1);
        }
    }
    sne(context) {
        let branch = false;
        if (context.instruction.modifier === IInstruction_1.ModifierType.I) {
            return this.sneI(context);
        }
        context.operands.forEach(p => {
            if (p.source.address !== p.destination.address) {
                branch = true;
            }
        });
        if (branch) {
            context.task.instructionPointer = context.core.wrap(context.task.instructionPointer + 1);
        }
    }
    sneI(context) {
        const ai = context.aInstruction;
        const bi = context.bInstruction;
        if (ai.opcode !== bi.opcode ||
            ai.modifier !== bi.modifier ||
            ai.aOperand.mode !== bi.aOperand.mode ||
            ai.aOperand.address !== bi.aOperand.address ||
            ai.bOperand.mode !== bi.bOperand.mode ||
            ai.bOperand.address !== bi.bOperand.address) {
            context.task.instructionPointer = context.core.wrap(context.task.instructionPointer + 1);
        }
    }
    slt(context) {
        let branch = true;
        context.operands.forEach(p => {
            if (p.source.address >= p.destination.address) {
                branch = false;
            }
        });
        if (branch) {
            context.task.instructionPointer = context.core.wrap(context.task.instructionPointer + 1);
        }
    }
    spl(context) {
        if (context.instance.tasks.length < this.maxTasks) {
            context.instance.tasks.splice(context.instance.taskIndex, 0, {
                instructionPointer: context.core.wrap(context.aInstruction.address),
                instance: context.instance
            });
            context.instance.taskIndex = (context.instance.taskIndex + 1) % context.instance.tasks.length;
            this.publishTaskCount(context.instance, context.instance.tasks.length);
        }
    }
    nop(_) {
        // DO NOWT
    }
}
exports.Executive = Executive;
//# sourceMappingURL=Executive.js.map