"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Simulator = void 0;
const IMessage_1 = require("./interface/IMessage");
const Defaults_1 = require("./Defaults");
const clone = require("clone");
class Simulator {
    constructor(core, loader, fetcher, decoder, executive, endCondition, optionValidator, publisher) {
        this.core = core;
        this.loader = loader;
        this.fetcher = fetcher;
        this.decoder = decoder;
        this.executive = executive;
        this.endCondition = endCondition;
        this.optionValidator = optionValidator;
        this.publisher = publisher;
        this.initState();
    }
    initState() {
        this.state = {
            cycle: 0,
            instances: [],
            warriorIndex: 0,
            options: null
        };
    }
    publishInitialise(state) {
        this.publisher.queue({
            type: IMessage_1.MessageType.CoreInitialise,
            payload: {
                state: clone(state)
            }
        });
    }
    publishNextExecution() {
        this.publisher.queue({
            type: IMessage_1.MessageType.NextExecution,
            payload: this.fetcher.getNextExecution(this.state)
        });
    }
    initialise(options, warriors) {
        this.publisher.clear();
        this.initState();
        const defaultedOptions = Object.assign({}, Defaults_1.default, options);
        this.optionValidator.validate(defaultedOptions, warriors.length);
        this.state.options = defaultedOptions;
        this.core.initialise(defaultedOptions);
        this.state.instances = this.loader.load(warriors, defaultedOptions);
        this.publishInitialise(this.state);
        this.publishNextExecution();
        this.publisher.publish();
    }
    run() {
        const remainingSteps = this.state.options.maximumCycles - this.state.cycle;
        return this.step(remainingSteps);
    }
    step(steps) {
        let result = this.endCondition.check(this.state);
        if (result) {
            return result;
        }
        result = this.multiStep(steps);
        this.publisher.publish();
        return result;
    }
    multiStep(steps) {
        if (!steps || steps < 1) {
            steps = 1;
        }
        if (this.state.cycle === 0) {
            this.publisher.queue({
                type: IMessage_1.MessageType.RoundStart,
                payload: {}
            });
        }
        for (let i = 0; i < steps; i++) {
            const result = this.singleStep();
            if (result) {
                return result;
            }
        }
        this.publishNextExecution();
        return null;
    }
    singleStep() {
        let context = this.fetcher.fetch(this.state, this.core);
        context = this.decoder.decode(context);
        context.command.apply(this.executive, [context]);
        this.state.cycle += 1;
        return this.endCondition.check(this.state);
    }
    getState() {
        return clone(this.state);
    }
}
exports.Simulator = Simulator;
//# sourceMappingURL=Simulator.js.map