"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WarriorLoader = void 0;
const IInstruction_1 = require("./interface/IInstruction");
const IToken_1 = require("../parser/interface/IToken");
const TokenStream_1 = require("../parser/TokenStream");
const WarriorInstance_1 = require("./WarriorInstance");
const IMessage_1 = require("./interface/IMessage");
class WarriorLoader {
    constructor(core, publisher) {
        this.core = core;
        this.publisher = publisher;
    }
    load(address, warrior, id) {
        const { source } = warrior;
        this.stream = new TokenStream_1.TokenStream(source.tokens, source.messages);
        this.address = address;
        this.instance = new WarriorInstance_1.WarriorInstance();
        this.instance.warrior = warrior;
        if (typeof this.instance.warrior.internalId === 'undefined') {
            this.instance.warrior.internalId = id;
        }
        this.instance.name = source.metaData.name;
        this.instance.author = source.metaData.author;
        this.instance.strategy = source.metaData.strategy;
        this.loadProcess(address);
        this.readInstructions();
        return this.instance;
    }
    readInstructions() {
        while (!this.stream.eof()) {
            const next = this.stream.peek();
            if (next.category === IToken_1.TokenCategory.Opcode) {
                this.core.setAt(this.instance.tasks[0], this.address, this.readInstruction());
                this.address += 1;
            }
            else if (next.category === IToken_1.TokenCategory.Preprocessor) {
                this.instance.tasks[0].instructionPointer = this.readOrg();
            }
            else {
                this.stream.readToEOL();
            }
        }
    }
    readInstruction() {
        const parseInstruction = this.readParseInstruction();
        const instruction = {
            address: this.address,
            opcode: this.getOpcode(parseInstruction),
            modifier: this.getModifier(parseInstruction),
            aOperand: this.getOperand(parseInstruction.aOperand),
            bOperand: this.getOperand(parseInstruction.bOperand)
        };
        return instruction;
    }
    readOrg() {
        this.stream.read(); // ORG
        const token = this.stream.read();
        this.stream.readToEOL();
        return this.address + parseInt(token.lexeme, 10);
    }
    readParseInstruction() {
        const result = {
            opcode: this.stream.read(),
            modifier: this.stream.read(),
            aOperand: {
                mode: this.stream.read(),
                address: this.stream.read()
            },
            comma: this.stream.read(),
            bOperand: {
                mode: this.stream.read(),
                address: this.stream.read()
            }
        };
        this.stream.readToEOL();
        return result;
    }
    getOpcode(instruction) {
        switch (instruction.opcode.lexeme) {
            case 'DAT':
                return IInstruction_1.OpcodeType.DAT;
            case 'MOV':
                return IInstruction_1.OpcodeType.MOV;
            case 'ADD':
                return IInstruction_1.OpcodeType.ADD;
            case 'SUB':
                return IInstruction_1.OpcodeType.SUB;
            case 'MUL':
                return IInstruction_1.OpcodeType.MUL;
            case 'DIV':
                return IInstruction_1.OpcodeType.DIV;
            case 'MOD':
                return IInstruction_1.OpcodeType.MOD;
            case 'JMP':
                return IInstruction_1.OpcodeType.JMP;
            case 'JMZ':
                return IInstruction_1.OpcodeType.JMZ;
            case 'JMN':
                return IInstruction_1.OpcodeType.JMN;
            case 'DJN':
                return IInstruction_1.OpcodeType.DJN;
            case 'CMP':
                return IInstruction_1.OpcodeType.CMP;
            case 'SEQ':
                return IInstruction_1.OpcodeType.SEQ;
            case 'SNE':
                return IInstruction_1.OpcodeType.SNE;
            case 'SLT':
                return IInstruction_1.OpcodeType.SLT;
            case 'SPL':
                return IInstruction_1.OpcodeType.SPL;
            default:
                return IInstruction_1.OpcodeType.NOP;
        }
    }
    getModifier(instruction) {
        switch (instruction.modifier.lexeme) {
            case '.A':
                return IInstruction_1.ModifierType.A;
            case '.B':
                return IInstruction_1.ModifierType.B;
            case '.BA':
                return IInstruction_1.ModifierType.BA;
            case '.F':
                return IInstruction_1.ModifierType.F;
            case '.I':
                return IInstruction_1.ModifierType.I;
            case '.X':
                return IInstruction_1.ModifierType.X;
            default:
                return IInstruction_1.ModifierType.AB;
        }
    }
    getOperand(operand) {
        const result = {
            mode: operand.mode.lexeme,
            address: this.core.wrap(parseInt(operand.address.lexeme, 10))
        };
        return result;
    }
    loadProcess(startAddress) {
        this.instance.tasks.push({
            instructionPointer: startAddress,
            instance: this.instance
        });
        this.instance.startAddress = startAddress;
        this.instance.taskIndex = 0;
        const payload = {
            warriorId: this.instance.warrior.internalId,
            taskCount: 1
        };
        if (this.instance.warrior.data) {
            payload['warriorData'] = this.instance.warrior.data;
        }
        this.publisher.queue({
            type: IMessage_1.MessageType.TaskCount,
            payload
        });
    }
}
exports.WarriorLoader = WarriorLoader;
//# sourceMappingURL=WarriorLoader.js.map